# -*- coding: utf-8 -*-
# Author: Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Windows tests."""

from itertools import count
from threading import Thread

# pylint: disable=F0401
from pywintypes import error
from win32pipe import CallNamedPipe
# pylint: enable=F0401

from mocker import MATCH, Mocker

from twisted.internet import defer, reactor
from twisted.trial.unittest import TestCase
from twisted.spread.pb import PBClientFactory, PBServerFactory
from ubuntu_sso.main.windows import (
    blocking,
    CredentialsManagement,
    CredentialsManagementClient,
    ListeningPortPipeService,
    SSOCredentials,
    SSOCredentialsClient,
    SSOLogin,
    SSOLoginClient,
    UbuntuSSORoot)

# because we are using twisted we have java like names C0103 and
# the issues that mocker brings with it like W0104
# pylint: disable=C0103,W0104


class SaveProtocolServerFactory(PBServerFactory):
    """A PBServerFactory that saves the latest connected client."""

    protocolInstance = None

    def clientConnectionMade(self, protocol):
        """Keep track of the given protocol."""
        self.protocolInstance = protocol


class SSOLoginTestCase(TestCase):
    """Test the login class."""

    def setUp(self):
        """Setup tests."""
        super(SSOLoginTestCase, self).setUp()
        self.mocker = Mocker()
        self.root = self.mocker.mock()
        self.except_to_errdict = self.mocker.replace(
                                        'ubuntu_sso.main.except_to_errdict')
        self.login = SSOLogin(None)
        self.login.root = self.root
        # start pb
        self.sso_root = UbuntuSSORoot(self.login, None, None)
        self.server_factory = SaveProtocolServerFactory(self.sso_root)
        # pylint: disable=E1101
        self.listener = reactor.listenTCP(0, self.server_factory)
        self.client_factory = PBClientFactory()
        self.connector = reactor.connectTCP('localhost',
                                            self.listener.getHost().port,
                                            self.client_factory)
        # pylint: enable=E1101

    def tearDown(self):
        """Clean reactor."""
        if self.server_factory.protocolInstance is not None:
            self.server_factory.protocolInstance.transport.loseConnection()
        return defer.gatherResults([self._tearDownServer(),
                                    self._tearDownClient()])

    def _tearDownServer(self):
        """Teardown the server."""
        return defer.maybeDeferred(self.listener.stopListening)

    def _tearDownClient(self):
        """Tear down the client."""
        self.connector.disconnect()
        return defer.succeed(None)

    @defer.inlineCallbacks
    def _get_client(self):
        """Get the client."""
        # request the remote object and create a client
        root = yield self.client_factory.getRootObject()
        remote = yield root.callRemote('get_sso_login')
        client = SSOLoginClient(remote)
        yield client.register_to_signals()
        # set the cb
        for signal in ['on_captcha_generated_cb',
                       'on_captcha_generation_error_cb',
                       'on_user_registered_cb',
                       'on_user_registration_error_cb',
                       'on_logged_in_cb',
                       'on_login_error_cb',
                       'on_user_not_validated_cb',
                       'on_email_validated_cb',
                       'on_email_validation_error_cb',
                       'on_password_reset_token_sent_cb',
                       'on_password_reset_error_cb',
                       'on_password_changed_cb',
                       'on_password_change_error_cb']:
            setattr(client, signal, self.mocker.mock())
        defer.returnValue(client)

    def test_emit_captcha_generated(self):
        """Test that the cb was called."""
        app_name = 'app'
        result = 'result'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_captcha_generated_cb(app_name, result)
            self.mocker.replay()
            self.login.emit_captcha_generated(app_name, result)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_captcha_generation_error(self):
        """Test that the cb was called."""
        app_name = 'app'
        error_value = 'error'
        raised_error = self.mocker.mock()

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            raised_error.value
            self.mocker.result(error_value)
            self.except_to_errdict(error_value)
            self.mocker.result(error_value)
            client.on_captcha_generation_error_cb(app_name, error_value)
            self.mocker.replay()
            self.login.emit_captcha_generation_error(app_name, raised_error)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_generate_captcha(self):
        """Test the call from the client."""
        app_name = 'app'
        filename = 'file'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.generate_captcha(app_name, filename, blocking,
                                   self.login.emit_captcha_generated,
                                   self.login.emit_captcha_generation_error)
            self.mocker.replay()
            yield client.generate_captcha(app_name, filename)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_emit_user_registered(self):
        """Test that the cb was called."""
        app_name = 'app'
        result = 'result'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_user_registered_cb(app_name, result)
            self.mocker.replay()
            self.login.emit_user_registered(app_name, result)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_user_registration_error(self):
        """Test that the cb was called."""
        app_name = 'app'
        error_value = 'error'
        raised_error = self.mocker.mock()

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            raised_error.value
            self.mocker.result(error_value)
            self.except_to_errdict(error_value)
            self.mocker.result(error_value)
            client.on_user_registration_error_cb(app_name, error_value)
            self.mocker.replay()
            self.login.emit_user_registration_error(app_name, raised_error)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_register_user(self):
        """Test the call from the client."""
        app_name = 'app'
        email = 'email'
        password = 'password'
        displayname = 'name'
        captcha_id = 'captcha_id'
        captcha_solution = 'captcha_solution'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.register_user(app_name, email, password, displayname,
                                    captcha_id, captcha_solution, blocking,
                                    self.login.emit_user_registered,
                                    self.login.emit_user_registration_error)
            self.mocker.replay()
            yield client.register_user(app_name, email, password, displayname,
                                       captcha_id, captcha_solution)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_emit_logged_in(self):
        """Test that the cb was called."""
        app_name = 'app'
        result = 'result'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_logged_in_cb(app_name, result)
            self.mocker.replay()
            self.login.emit_logged_in(app_name, result)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_login_error(self):
        """Test that the db was called."""
        app_name = 'app'
        error_value = 'error'
        raised_error = self.mocker.mock()

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            raised_error.value
            self.mocker.result(error_value)
            self.except_to_errdict(error_value)
            self.mocker.result(error_value)
            client.on_login_error_cb(app_name, error_value)
            self.mocker.replay()
            self.login.emit_login_error(app_name, raised_error)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_user_not_validated(self):
        """Test that the cb was called."""
        app_name = 'app'
        result = 'result'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_user_not_validated_cb(app_name, result)
            self.mocker.replay()
            self.login.emit_user_not_validated(app_name, result)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_login(self):
        """Test the call from the client."""
        app_name = 'app'
        email = 'email'
        password = 'password'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.login(app_name, email, password, blocking,
                            self.login.emit_logged_in,
                            self.login.emit_login_error,
                            self.login.emit_user_not_validated)
            self.mocker.replay()
            yield client.login(app_name, email, password)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_emit_email_validated(self):
        """Test the cb was called."""
        app_name = 'app'
        result = 'result'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_email_validated_cb(app_name, result)
            self.mocker.replay()
            self.login.emit_email_validated(app_name, result)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_email_validation_error(self):
        """Test the cb was called."""
        app_name = 'app'
        error_value = 'error'
        raised_error = self.mocker.mock()

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            raised_error.value
            self.mocker.result(error_value)
            self.except_to_errdict(error_value)
            self.mocker.result(error_value)
            client.on_email_validation_error_cb(app_name, error_value)
            self.mocker.replay()
            self.login.emit_email_validation_error(app_name, raised_error)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_validate_email(self):
        """Test the client calll."""
        app_name = 'app'
        email = 'email'
        password = 'password'
        email_token = 'token'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.validate_email(app_name, email, password, email_token,
                                 blocking, self.login.emit_email_validated,
                                 self.login.emit_email_validation_error)
            self.mocker.replay()
            yield client.validate_email(app_name, email, password, email_token)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_emit_password_reset_token_sent(self):
        """Test the cb was called."""
        app_name = 'app'
        result = 'result'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_password_reset_token_sent_cb(app_name, result)
            self.mocker.replay()
            self.login.emit_password_reset_token_sent(app_name, result)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_password_reset_error(self):
        """Test the cb was called."""
        app_name = 'app'
        error_value = 'error'
        raised_error = self.mocker.mock()

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            raised_error.value
            self.mocker.result(error_value)
            self.except_to_errdict(error_value)
            self.mocker.result(error_value)
            client.on_password_reset_error_cb(app_name, error_value)
            self.mocker.replay()
            self.login.emit_password_reset_error(app_name, raised_error)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_request_password_reset_token(self):
        """Test the client call."""
        app_name = 'app'
        email = 'email'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.request_password_reset_token(app_name, email, blocking,
                                self.login.emit_password_reset_token_sent,
                                self.login.emit_password_reset_error)
            self.mocker.replay()
            client.request_password_reset_token(app_name, email)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_emit_password_changed(self):
        """Test the cb was called."""
        app_name = 'app'
        result = 'result'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_password_changed_cb(app_name, result)
            self.mocker.replay()
            self.login.emit_password_changed(app_name, result)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_password_change_error(self):
        """Test the cb was called."""
        app_name = 'app'
        error_value = 'error'
        raised_error = self.mocker.mock()

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            raised_error.value
            self.mocker.result(error_value)
            self.except_to_errdict(error_value)
            self.mocker.result(error_value)
            client.on_password_change_error(app_name, error_value)
            self.mocker.replay()
            self.login.emit_password_change_error(app_name, raised_error)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_set_new_password(self):
        """Test the client call."""
        app_name = 'app'
        email = 'email'
        token = 'token'
        new_password = 'password'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.set_new_password(app_name, email, token, new_password,
                                   blocking, self.login.emit_password_changed,
                                   self.login.emit_password_change_error)
            self.mocker.replay()
            yield client.set_new_password(app_name, email, token, new_password)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d


class SSOCredentialsTestCase(TestCase):
    """Test the credentials class."""

    def setUp(self):
        """Set up tests."""
        super(SSOCredentialsTestCase, self).setUp()
        self.mocker = Mocker()
        self.root = self.mocker.mock()
        self.except_to_errdict = self.mocker.replace(
                                        'ubuntu_sso.main.except_to_errdict')
        self.creds = SSOCredentials(None)
        self.creds.root = self.root
        # start pb
        self.sso_root = UbuntuSSORoot(None, self.creds, None)
        self.server_factory = SaveProtocolServerFactory(self.sso_root)
        # pylint: disable=E1101
        self.listener = reactor.listenTCP(0, self.server_factory)
        self.client_factory = PBClientFactory()
        self.connector = reactor.connectTCP('localhost',
                                            self.listener.getHost().port,
                                            self.client_factory)
        # pylint: enable=E1101

    def tearDown(self):
        """Clean reactor."""
        if self.server_factory.protocolInstance is not None:
            self.server_factory.protocolInstance.transport.loseConnection()
        return defer.gatherResults([self._tearDownServer(),
                                    self._tearDownClient()])

    def _tearDownServer(self):
        """Teardown the server."""
        return defer.maybeDeferred(self.listener.stopListening)

    def _tearDownClient(self):
        """Tear down the client."""
        self.connector.disconnect()
        return defer.succeed(None)

    @defer.inlineCallbacks
    def _get_client(self):
        """Get the client."""
        # request the remote object and create a client
        root = yield self.client_factory.getRootObject()
        remote = yield root.callRemote('get_sso_credentials')
        client = SSOCredentialsClient(remote)
        yield client.register_to_signals()
        # set the cb
        for signal in ['on_authorization_denied_cb',
                       'on_credentials_found_cb',
                       'on_credentials_error_cb']:
            setattr(client, signal, self.mocker.mock())
        defer.returnValue(client)

    def test_emit_authorization_denied(self):
        """Test that the cb is executed."""
        app_name = 'app'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_authorization_denied_cb(app_name)
            self.mocker.replay()
            self.creds.emit_authorization_denied(app_name)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_credentials_found(self):
        """Test that the cb was executed."""
        app_name = 'app'
        credentials = 'cred'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_credentials_found_cb(app_name, credentials)
            self.mocker.replay()
            self.creds.emit_credentials_found(app_name, credentials)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_credentials_error(self):
        """Test that the cb was executed."""
        app_name = 'app'
        raised_error = 'error'
        detail = 'detail'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            client.on_credentials_error_cb(app_name, raised_error, detail)
            self.mocker.replay()
            self.creds.emit_credentials_error(app_name, raised_error, detail)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_find_credentials_no_kword(self):
        """Ensure that the root is called."""
        app_name = 'app'
        callback = lambda: None
        errback = lambda: None

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.find_credentials(app_name, MATCH(callable),
                                       MATCH(callable))
            self.mocker.replay()
            yield client.find_credentials(app_name, callback, errback)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_find_credentials_(self):
        """Ensure that the root is called."""
        app_name = 'app'
        callback = lambda: None
        errback = lambda: None

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.find_credentials(app_name, MATCH(callable),
                                       MATCH(callable))
            self.mocker.replay()
            yield client.find_credentials(app_name, callback=callback,
                                          errback=errback)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_login_or_register_to_get_credentials(self):
        """Ensure that the root is called."""
        app_name = 'app'
        terms = 'terms'
        help_txt = 'help'
        window_id = 'window_id'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            # pylint: disable=W0212
            self.root.login_or_register_to_get_credentials(app_name, terms,
                                        help_txt, window_id,
                                        self.creds.emit_credentials_found,
                                        self.creds._process_error,
                                        self.creds.emit_authorization_denied)
            # pylint: enable=W0212
            self.mocker.replay()
            yield client.login_or_register_to_get_credentials(app_name, terms,
                                                          help_txt, window_id)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_login_to_get_credentials(self):
        """Ensure that the root is called."""
        app_name = 'app'
        help_txt = 'help'
        window_id = 'window_id'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            # pylint: disable=W0212
            self.root.login_to_get_credentials(app_name, help_txt, window_id,
                                        self.creds.emit_credentials_found,
                                        self.creds._process_error,
                                        self.creds.emit_authorization_denied)
            # pylint: enable=W0212
            self.mocker.replay()
            yield client.login_to_get_credentials(app_name, help_txt,
                                                  window_id)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_clear_token_no_kword(self):
        """Ensure that the root is called."""
        app_name = 'app'
        callback = lambda: None
        errback = lambda: None

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.clear_token(app_name, MATCH(callable), MATCH(callable))
            self.mocker.replay()
            yield client.clear_token(app_name, callback, errback)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_clear_token(self):
        """Ensure that the root is called."""
        app_name = 'app'
        callback = lambda: None
        errback = lambda: None

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.clear_token(app_name, MATCH(callable), MATCH(callable))
            self.mocker.replay()
            yield client.clear_token(app_name, callback=callback,
                                     errback=errback)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d


class CredentialsManagementTestCase(TestCase):
    """Test the management class."""

    def setUp(self):
        """Set up tests."""
        super(CredentialsManagementTestCase, self).setUp()
        self.mocker = Mocker()
        self.root = self.mocker.mock()
        self.except_to_errdict = self.mocker.replace(
                                        'ubuntu_sso.main.except_to_errdict')
        self.creds = CredentialsManagement(None, None)
        self.creds.root = self.root
        # start pb
        self.sso_root = UbuntuSSORoot(None, None, self.creds)
        self.server_factory = SaveProtocolServerFactory(self.sso_root)
        # pylint: disable=E1101
        self.listener = reactor.listenTCP(0, self.server_factory)
        self.client_factory = PBClientFactory()
        self.connector = reactor.connectTCP('localhost',
                                            self.listener.getHost().port,
                                            self.client_factory)
        # pylint: enable=E1101

    def tearDown(self):
        """Clean reactor."""
        if self.server_factory.protocolInstance is not None:
            self.server_factory.protocolInstance.transport.loseConnection()
        return defer.gatherResults([self._tearDownServer(),
                                    self._tearDownClient()])

    def _tearDownServer(self):
        """Teardown the server."""
        return defer.maybeDeferred(self.listener.stopListening)

    def _tearDownClient(self):
        """Tear down the client."""
        self.connector.disconnect()
        return defer.succeed(None)

    @defer.inlineCallbacks
    def _get_client(self):
        """Get the client."""
        # request the remote object and create a client
        root = yield self.client_factory.getRootObject()
        remote = yield root.callRemote('get_cred_manager')
        client = CredentialsManagementClient(remote)
        yield client.register_to_signals()
        # set the cb
        for signal in ['on_authorization_denied_cb',
                       'on_credentials_found_cb',
                       'on_credentials_not_found_cb',
                       'on_credentials_cleared_cb',
                       'on_credentials_stored_cb',
                       'on_credentials_error_cb']:
            setattr(client, signal, self.mocker.mock())
        defer.returnValue(client)

    def test_shutdown(self):
        """Test that root is called."""

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            # pylint: disable=W0104
            self.root.ref_count
            # pylint: enable=W0104
            self.mocker.result(1)
            self.root.shutdown()
            self.mocker.replay()
            yield client.shutdown()
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_emit_authorization_denied(self):
        """Test the callback is called."""
        app_name = 'app'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            # pylint: disable=W0104
            self.root.ref_count
            # pylint: enable=W0104
            self.mocker.result(1)
            self.root.ref_count = 0
            client.on_authorization_denied_cb(app_name)
            self.mocker.replay()
            self.creds.emit_authorization_denied(app_name)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_credentials_found(self):
        """Test the callback is called."""
        app_name = 'app'
        creds = 'creds'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            # pylint: disable=W0104
            self.root.ref_count
            # pylint: enable=W0104
            self.mocker.result(1)
            self.root.ref_count = 0
            client.on_credentials_found_cb(app_name, creds)
            self.mocker.replay()
            self.creds.emit_credentials_found(app_name, creds)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_credentials_not_found(self):
        """Test the callback is called."""
        app_name = 'app'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            # pylint: disable=W0104
            self.root.ref_count
            # pylint: enable=W0104
            self.mocker.result(1)
            self.root.ref_count = 0
            client.on_credentials_not_found_cb(app_name)
            self.mocker.replay()
            self.creds.emit_credentials_not_found(app_name)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_credentials_cleared(self):
        """Test the callback is called."""
        app_name = 'app'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            # pylint: disable=W0104
            self.root.ref_count
            # pylint: enable=W0104
            self.mocker.result(1)
            self.root.ref_count = 0
            client.on_credentials_cleared_cb(app_name)
            self.mocker.replay()
            self.creds.emit_credentials_cleared(app_name)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_credentials_stored(self):
        """Test the callback is called."""
        app_name = 'app'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            # pylint: disable=W0104
            self.root.ref_count
            # pylint: enable=W0104
            self.mocker.result(1)
            self.root.ref_count = 0
            client.on_credentials_stored_cb(app_name)
            self.mocker.replay()
            self.creds.emit_credentials_stored(app_name)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_emit_credentials_error(self):
        """Test the callback is called."""
        app_name = 'app'
        raised_error = 'error'

        @defer.inlineCallbacks
        def test_emit(client):
            """Actual test."""
            # pylint: disable=W0104
            self.root.ref_count
            # pylint: enable=W0104
            self.mocker.result(1)
            self.root.ref_count = 0
            client.on_credentials_error_cb(app_name, raised_error)
            self.mocker.replay()
            self.creds.emit_credentials_error(app_name, raised_error)
            yield client.unregister_to_signals()
            self.mocker.verify()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_emit)
        # pylint: enable=E1101
        return d

    def test_find_credentials(self):
        """Test that root is called."""
        app_name = 'app'
        args = 'args'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            # pylint: disable=W0212
            self.root.find_credentials(app_name, args, MATCH(callable),
                                       self.creds._process_failure)
            # pylint: enable=W0212
            self.root.shutdown()
            yield client.find_credentials(app_name, args)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_clear_credentials(self):
        """Test that root is called."""
        app_name = 'app'
        args = 'args'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            # pylint: disable=W0212
            self.root.clear_credentials(app_name, args, MATCH(callable),
                                        self.creds._process_failure)
            # pylint: enable=W0212
            self.mocker.replay()
            yield client.clear_credentials(app_name, args)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_store_credentials(self):
        """Test that root is called."""
        app_name = 'app'
        args = 'args'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            # pylint: disable=W0212
            self.root.store_credentials(app_name, args, MATCH(callable),
                               self.creds._process_failure)
            # pylint: enable=W0212
            self.mocker.replay()
            yield client.store_credentials(app_name, args)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_register(self):
        """Test that root is called."""
        app_name = 'app'
        args = 'args'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.register(app_name, args)
            self.mocker.replay()
            yield client.register(app_name, args)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d

    def test_login(self):
        """Test that root is called."""
        app_name = 'app'
        args = 'args'

        @defer.inlineCallbacks
        def test_execution(client):
            """Actual test."""
            self.root.login(app_name, args)
            self.mocker.replay()
            yield client.login(app_name, args)
            yield client.unregister_to_signals()

        d = self._get_client()
        # pylint: disable=E1101
        d.addCallback(test_execution)
        # pylint: enable=E1101
        return d


class TestListeningPortPipeService(TestCase):
    """Test that the service those expose the port number correctly."""

    def setUp(self):
        super(TestListeningPortPipeService, self).setUp()
        self.usernmae = 'mandel'
        self.port = 9099
        self.service = ListeningPortPipeService(self.usernmae, self.port,
                                                client_wait=500)

    def test_call_namedpipe(self):
        """Ensure that the correct port is returned."""
        t = Thread(target=self.service.start, name='Pipe service')
        t.start()
        port = int(CallNamedPipe(self.service.pipe_name, '', 512, 0))
        self.assertEqual(self.port, port)
        self.service.stop()
        t.join()

    def test_call_serveral_named_pipe(self):
        """Ensure that the server can handle lots of clients."""
        # imaging that 300 apps have integrated with u1
        number_threads = 300
        t = Thread(target=self.service.start, name='Pipe service')
        t.start()

        def call_named_pipe(number):
            """Actual test ran in a thread."""
            # with the current speed in which the pipe is created, the last
            # thread must raise an error
            if number < number_threads:
                port = int(CallNamedPipe(self.service.pipe_name, '', 512, 0))
                self.assertEqual(self.port, port)

        # create lost of threads that request the port number
        for i in count(1):
            if i <= number_threads:
                Thread(target=call_named_pipe, args=(i,),
                       name='Assert %s' % i).start()
            else:
                break
        self.service.stop()
        t.join()

    def test_call_stop_with_slow_pool_frec(self):
        """Ensure that when stop is called, we do not have issues."""
        self.service = ListeningPortPipeService(self.usernmae, self.port,
                                                client_wait=500,
                                                pool_frec=800000)
        t = Thread(target=self.service.start, name='Pipe service')
        t.start()
        self.service.stop()
        self.assertRaises(error, CallNamedPipe, self.service.pipe_name,
                          '', 512, 0)
