# -*- coding: utf-8 -*-
# Author: Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Controllers with the logic of the UI."""

import os
import tempfile

from PyQt4.QtGui import QMessageBox
from PyQt4.QtCore import QUrl
from twisted.internet.defer import inlineCallbacks, returnValue

from ubuntu_sso.logger import setup_logging
from ubuntu_sso.main.windows import UbuntuSSOClient
from ubuntu_sso.utils.ui import (
    CAPTCHA_SOLUTION_ENTRY,
    EMAIL1_ENTRY,
    EMAIL2_ENTRY,
    EMAIL_MISMATCH,
    EMAIL_INVALID,
    EXISTING_ACCOUNT_CHOICE_BUTTON,
    FORGOTTEN_PASSWORD_BUTTON,
    JOIN_HEADER_LABEL,
    NAME_ENTRY,
    PASSWORD1_ENTRY,
    PASSWORD2_ENTRY,
    PASSWORD_HELP,
    PASSWORD_MISMATCH,
    PASSWORD_TOO_WEAK,
    SET_UP_ACCOUNT_CHOICE_BUTTON,
    SET_UP_ACCOUNT_BUTTON,
    SIGN_IN_BUTTON,
    SUCCESS,
    SURNAME_ENTRY,
    TC_BUTTON,
    VERIFY_EMAIL_TITLE,
    VERIFY_EMAIL_CONTENT,
    YES_TO_TC,
    get_password_strength,
    is_min_required_password)


logger = setup_logging('ubuntu_sso.controllers')


class BackendController(object):
    """Represent a controller that talks with the sso backend."""

    def __init__(self):
        """Create a new instance."""
        self.root = None
        self.backends = []

    def __del__(self):
        """Clean the resources."""
        logger.info('Unregistering %s backends', len(self.backends))
        for backend in self.backends:
            backend.unregister_to_signals()
        if self.root is not None:
            logger.info('Disconnecting from root.')
            self.root.disconnect()

    @inlineCallbacks
    def get_backend(self):
        """Return the backend used by the controller."""
        # get the back end from the root
        if self.root is None:
            self.root = UbuntuSSOClient()
            self.root = yield self.root.connect()
        backend = self.root.sso_login
        yield backend.register_to_signals()
        self.backends.append(backend)
        returnValue(backend)


class ChooseSignInController(object):
    """Controlled to the ChooseSignIn view/widget."""

    def __init__(self, title='', existing_account_id=1,
                 new_account_id=2):
        """Create a new instance to manage the view."""
        self._title = title
        self._existing_account_id = existing_account_id
        self._new_account_id = new_account_id

    # use an ugly name just so have a simlar api as found in PyQt
    #pylint: disable=C0103
    def setupUi(self, view):
        """Perform the required actions to set up the ui."""
        view.setTitle(self._title)
        self._set_up_translated_strings(view)
        self._connect_buttons(view)
    #pylint: enable=C0103

    def _set_up_translated_strings(self, view):
        """Set the correct strings for the UI."""
        view.existing_account_button.setText(EXISTING_ACCOUNT_CHOICE_BUTTON)
        view.new_account_button.setText(SET_UP_ACCOUNT_CHOICE_BUTTON)

    def _connect_buttons(self, view):
        """Connect the buttons to the actions to perform."""
        view.existing_account_button.clicked.connect(
                                        lambda: self._set_next_existing(view))
        view.new_account_button.clicked.connect(
                                             lambda: self._set_next_new(view))

    def _set_next_existing(self, view):
        """Set the next id and fire signal."""
        view.next = self._existing_account_id
        view.wizard().next()

    def _set_next_new(self, view):
        """Set the next id and fire signal."""
        view.next = self._new_account_id
        view.wizard().next()


class CurrentUserController(BackendController):
    """Controller used in the view that is used to allow the signin."""

    def __init__(self, backend=None, title='', app_name='', message_box=None):
        """Create a new instance."""
        super(CurrentUserController, self).__init__()
        if message_box is None:
            message_box = QMessageBox
        self.message_box = message_box
        self._title = title
        self._app_name = app_name

    def _set_translated_strings(self, view):
        """Set the translated strings."""
        logger.debug('Setting tranlated strings.')
        view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        view.forgot_password_label.setText(FORGOTTEN_PASSWORD_BUTTON)
        view.sign_in_button.setText(SIGN_IN_BUTTON)

    def _connect_buttons(self, view, backend):
        """Connect the buttons to perform actions."""
        view.sign_in_button.clicked.connect(lambda: self.login(view, backend))
        # lets add call backs to be execute for the calls we are interested
        backend.on_login_error_cb = lambda app, error:\
                                        self.on_login_error(view, app, error)
        backend.on_logged_in_cb = lambda app, result:\
                                        self.on_logged_in(view, app, result)

    def login(self, view, backend):
        """Perform the login using the backend."""
        logger.debug('Trying to log in using %s', backend)
        # grab the data from the view and call the backend
        d = backend.login(self._app_name, view.email, view.password)
        d.addErrback(lambda e: self.on_login_error(view, self._app_name, e))

    def on_login_error(self, view, app_name, error):
        """There was an error when login in."""
        # let the user know
        logger.error('Got error when login %s, error: %s', app_name, error)
        self.message_box.critical(view, app_name, error)

    def on_logged_in(self, view, app_name, result):
        """We managed to log in."""
        logger.info('Logged in for %s', app_name)
        self.message_box.information(view, app_name, SUCCESS)
        view.wizard.close()

    # use an ugly name just so have a simlar api as found in PyQt
    #pylint: disable=C0103
    @inlineCallbacks
    def setupUi(self, view):
        """Setup the view."""
        backend = yield self.get_backend()
        view.setTitle(self._title)
        self._set_translated_strings(view)
        self._connect_buttons(view, backend)
    #pylint: enable=C0103


class SetUpAccountController(BackendController):
    """Conroller for the setup account view."""

    def __init__(self, tos_id=0, validation_id=1, app_name='',
                 help_message='', message_box=None):
        """Create a new instance."""
        super(SetUpAccountController, self).__init__()
        if message_box is None:
            message_box = QMessageBox
        self.message_box = message_box
        self._tos_id = tos_id
        self._validation_id = validation_id
        self._app_name = app_name
        self._help_message = help_message

    def _set_translated_strings(self, view):
        """Set the different gettext translated strings."""
        logger.debug('Setting tranlated strings.')
        # set the translated string
        view.name_edit.setPlaceholderText(NAME_ENTRY)
        view.last_name_edit.setPlaceholderText(SURNAME_ENTRY)
        view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        view.confirm_email_edit.setPlaceholderText(EMAIL2_ENTRY)
        view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        view.confirm_password_edit.setPlaceholderText(PASSWORD2_ENTRY)
        view.password_info_label.setText(PASSWORD_HELP)
        view.captcha_solution_edit.setPlaceholderText(CAPTCHA_SOLUTION_ENTRY)
        view.terms_and_conditions_check_box.setText(YES_TO_TC)
        view.terms_and_conditions_button.setText(TC_BUTTON)
        view.set_up_button.setText(SET_UP_ACCOUNT_BUTTON)

    def _set_line_edits_validations(self, view):
        """Set the validations to be performed on the edits."""
        view.set_line_edit_validation_rule(view.email_edit,
                                           self.is_correct_email)
        # set the validation rule for the email confirmation
        view.set_line_edit_validation_rule(view.confirm_email_edit,
                         lambda x: self.is_correct_email_confirmation(x, view))
        # connect the changed text of the password to trigger a changed text
        # in the confirm so that the validation is redone
        view.email_edit.textChanged.connect(
                                      view.confirm_email_edit.textChanged.emit)
        view.set_line_edit_validation_rule(view.password_edit,
                                           is_min_required_password)
        view.set_line_edit_validation_rule(view.confirm_password_edit,
                      lambda x: self.is_correct_password_confirmation(x, view))
        # same as the above case, lets connect a signal to a signal
        view.password_edit.textChanged.connect(
                                   view.confirm_password_edit.textChanged.emit)

    def _connect_ui_elements(self, view, backend):
        """Set the connection of signals."""
        view.terms_and_conditions_button.clicked.connect(
                                               lambda: self.set_next_tos(view))
        view.set_up_button.clicked.connect(lambda: self.set_next_validation(
                                                                view, backend))
        view.password_edit.textChanged.connect(
                              lambda x: self.update_password_strength(x, view))
        view.terms_and_conditions_check_box.stateChanged.connect(
                                                 view.set_up_button.setEnabled)
        view.captcha_refresh_label.linkActivated.connect(lambda url:\
                                          self._refresh_captcha(view, backend))
        # set the callbacks for the captcha generation
        backend.on_captcha_generated_cb = lambda app, result:\
                                self.on_captcha_generated(view, app, result)
        backend.on_captcha_generation_error_cb = lambda app, error:\
                            self.on_captcha_generation_error(view, app, error)
        backend.on_user_registration_error_cb = lambda app, error:\
                            self.on_user_registration_error(view, app, error)
        return backend

    def _refresh_captcha(self, view, backend):
        """Refresh the captcha image shown in the ui."""
        # lets clean behind us, do we have the old file arround?
        old_file = None
        if view.captcha_file and os.path.exists(view.captcha_file):
            old_file = view.captcha_file
        fd = tempfile.TemporaryFile(mode='r')
        file_name = fd.name
        view.captcha_file = file_name
        d = backend.generate_captcha(self._app_name, file_name)
        if old_file:
            d.addCallback(lambda x: os.unlink(old_file))
        d.addErrback(lambda e: self.on_captcha_generation_error(view,
                                                                self._app_name,
                                                                e))

    def _set_titles(self, view):
        """Set the diff titles of the view."""
        view.setTitle(JOIN_HEADER_LABEL % {'app_name': self._app_name})
        view.setSubTitle(self._help_message)

    def on_captcha_generated(self, view, app_name, result):
        """A new image was generated."""
        view.captcha_image = result

    def on_captcha_generation_error(self, view, app_name, error):
        """An error ocurred."""
        self.message_box.critical(view, app_name, str(error))

    def on_user_registration_error(self, view, app_name, error):
        """Let the user know we could not register."""
        # error are returned as a dict with the data we want to show.
        self.message_box.critical(view, error['errtype'], error['email'])

    def set_next_tos(self, view):
        """Set the tos page as the next one."""
        view.next = self._tos_id
        view.wizard().next()

    def validate_form(self, view):
        """Validate the info of the form and return an error."""
        if not self.is_correct_email(view.email):
            self.message_box.critical(view, self._app_name, EMAIL_INVALID)
        if view.email_edit.text() != view.confirm_email_edit.text():
            self.message_box.critical(view, self._app_name, EMAIL_MISMATCH)
            return False
        if not is_min_required_password(str(view.password_edit.text())):
            self.message_box.critical(view, self._app_name, PASSWORD_TOO_WEAK)
            return False
        if view.password_edit.text() != view.confirm_password_edit.text():
            self.message_box.critical(view, self._app_name, PASSWORD_MISMATCH)
            return False
        return True

    def set_next_validation(self, view, backend):
        """Set the validation as the next page."""
        # validate the current info of the form, try to perform the action
        # to register the user, and then move foward
        if self.validate_form(view):
            backend.register_user(self._app_name, view.email, view.password,
                                  view.captcha_id, view.captcha_solution)
            view.next = self._validation_id
            view.wizard().next()

    def update_password_strength(self, password, view):
        """Callback used to update the password strenght UI code."""
        # get the strengh and then according to it color the frames
        strengh = get_password_strength(password)
        logger.info('Password strengh is %s', strengh)
        view.set_strenght_level(strengh, password)

    def is_correct_email(self, email_address):
        """Return if the email is correct."""
        return '@' in email_address

    def is_correct_email_confirmation(self, email_address, view):
        """Return that the email is the same."""
        return view.email_edit.text() == email_address

    def is_correct_password_confirmation(self, password, view):
        """Return that the passwords are correct."""
        return view.password_edit.text() == password

    #pylint: disable=C0103
    @inlineCallbacks
    def setupUi(self, view):
        """Setup the view."""
        # request the backend to be used with the ui
        backend = yield self.get_backend()
        self._connect_ui_elements(view, backend)
        self._refresh_captcha(view, backend)
        self._set_titles(view)
        self._set_translated_strings(view)
        self._set_line_edits_validations(view)
    #pylint: enable=C0103


class TosController(object):
    """Controller used for the tos page."""

    def __init__(self, title='', subtitle='', tos_url='http://www.ubuntu.com'):
        """Create a new instance."""
        self._title = title
        self._subtitle = subtitle
        self._tos_url = tos_url

    #pylint: disable=C0103
    def setupUi(self, view):
        """Set up the ui."""
        view.setTitle(self._title)
        view.setSubTitle(self._subtitle)
        # load the tos page
        view.webkit.load(QUrl(self._tos_url))
    #pylint: enable=C0103


class EmailVerificationController(object):
    """Controller used for the verification page."""

    def _set_translated_strings(self, view):
        """Set the trnaslated strings."""
        view.verification_code_edit.setPlaceholderText(VERIFY_EMAIL_TITLE)

    def _connect_ui_elements(self, view):
        """Set the connection of signals."""
        view.next_button.clicked.connect(lambda: self.next_page(view))

    def _set_titles(self, view):
        """Set the different titles."""
        view.setTitle(VERIFY_EMAIL_TITLE)
        view.setSubTitle(VERIFY_EMAIL_CONTENT)

    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        self._set_titles(view)
        self._set_translated_strings(view)
        self._connect_ui_elements(view)
    #pylint: enable=C0103

    def next_page(self, view):
        """Call the next action."""
        view.wizard().next()
