# -*- coding: utf-8 -*-
# Author: Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Qt implementation of the UI."""

from PyQt4.QtGui import (
    QApplication,
    QColor,
    QCursor,
    QHBoxLayout,
    QPalette,
    QPixmap,
    QPushButton,
    QStyle,
    QWizard,
    QWizardPage)

from ubuntu_sso.logger import setup_logging
# pylint: disable=F0401,E0611
from ubuntu_sso.qt.choose_sign_in_ui import Ui_ChooseSignInPage
from ubuntu_sso.qt.current_user_sign_in_ui import Ui_CurrentUserSignInPage
from ubuntu_sso.qt.email_verification_ui import Ui_EmailVerificationPage
from ubuntu_sso.qt.setup_account_ui import Ui_SetUpAccountPage
from ubuntu_sso.qt.terms_and_conditions_ui import Ui_TosPage
# pylint: enable=F0401,E0611
from ubuntu_sso.qt.controllers import (
    ChooseSignInController,
    CurrentUserController,
    EmailVerificationController,
    SetUpAccountController,
    TosController)


logger = setup_logging('ubuntu_sso.gui')

# colors used to define the password stenght
WEAK_COLOR = QColor(220, 20, 60)
MEDIUM_COLOR = QColor(255, 215, 0)
STRONG_COLOR = QColor(50, 205, 50)


class ChooseSignInPage(QWizardPage):
    """Widget that allows the user to choose how to sign in."""

    def __init__(self, ui, controller, parent=None):
        """Create a new widget to be used."""
        QWizardPage.__init__(self, parent)
        self.ui = ui
        self.controller = controller
        ui.setupUi(self)
        controller.setupUi(self)
        self.next = -1

    # pylint: disable=C0103
    def nextId(self):
        """Provide the next id."""
        return self.next
    # pylint: enable=C0103

    # allow to access to the different useful children
    @property
    def existing_account_button(self):
        """Return the button used to sign in using an existing account."""
        return self.ui.existing_account_button

    @property
    def new_account_button(self):
        """Return the button used to sign in with a new account."""
        return self.ui.setup_account_button


class CurrentUserSignInPage(QWizardPage):
    """Widget that allows to get the data of user to sign in."""

    def __init__(self, ui, controller, parent=None):
        """Create a new widget to be used."""
        QWizardPage.__init__(self, parent)
        self.ui = ui
        self.controller = controller
        self.ui.setupUi(self)
        self.controller.setupUi(self)

    # allow to access to the different useful data
    @property
    def email(self):
        """Return the email data in the edit field."""
        return str(self.ui.email_edit.text())

    @property
    def email_edit(self):
        """Return the email edit that is used in the view."""
        return self.ui.email_edit

    @property
    def password(self):
        """Return the password data in the edit field."""
        return str(self.ui.password_edit.text())

    @property
    def password_edit(self):
        """Return the password edit used in the view."""
        return self.ui.password_edit

    @property
    def forgot_password_label(self):
        """Return the forgot password label."""
        return self.ui.forgot_label

    @property
    def sign_in_button(self):
        """Return the sign in button."""
        return self.ui.sign_in_button


class EmailVerificationPage(QWizardPage):
    """Widget used to input the email verification code."""

    def __init__(self, ui, controller, parent=None):
        """Create a new widget to be used."""
        QWizardPage.__init__(self, parent)
        self.ui = ui
        self.ui.setupUi(self)
        self.controller = controller
        self.controller.setupUi(self)

    @property
    def verification_code(self):
        """Return the content of the verification code edit."""
        return str(self.ui.verification_code_edit)

    @property
    def verification_code_edit(self):
        """Return the edit used for the verification code."""
        return self.ui.verification_code_edit

    @property
    def next_button(self):
        """Return the button that move to the next stage."""
        return self.ui.next_button


class TosPage(QWizardPage):
    """Widget used to show the tos."""

    def __init__(self, ui, controller, parent=None):
        """Create a new instance."""
        QWizardPage.__init__(self, parent)
        self.ui = ui
        self.ui.setupUi(self)
        self.controller = controller
        self.controller.setupUi(self)

    # pylint: disable=C0103
    def nextId(self):
        """Return the next page id."""
        return -1
    # pylint: enable=C0103

    @property
    def webkit(self):
        """Return the webkit widget used to load the terms."""
        return self.ui.terms_webkit


class EnhancedLineEdit(object):
    """Represents an enhanced lineedit.

    This class works on an already added lineedit to the widget so
    that we are just adding extra items to it.
    """

    def __init__(self, line_edit, valid_cb=lambda x: False):
        """Create an instance."""
        self._line_edit = line_edit
        layout = QHBoxLayout(self._line_edit)
        layout.setMargin(0)
        self._line_edit.setLayout(layout)
        self.valid_cb = valid_cb
        layout.addStretch()
        self.clear_button = QPushButton(self._line_edit)
        layout.addWidget(self.clear_button)
        self.clear_button.setMinimumSize(16, 16)
        self.clear_button.setVisible(False)
        self.clear_button.setFlat(True)
        self.clear_button.setCursor(QCursor(0))
        self.clear_button.setIcon(QApplication.style().standardIcon(
                                                QStyle.SP_MessageBoxWarning))
        # connect the change of text to the caption that will check if the
        # text is valid and if the icon should be shown.
        self._line_edit.textChanged.connect(self.show_button)

    def show_button(self, string):
        """Decide if we show the button or not."""
        if not self.valid_cb(string):
            self.clear_button.setVisible(True)
        else:
            self.clear_button.setVisible(False)


class SetupAccountPage(QWizardPage):
    """Widget used to create a new account."""

    def __init__(self, ui, controller, parent=None):
        """Create a new widget to be used."""
        QWizardPage.__init__(self, parent)
        self._enhanced_edits = {}
        self.ui = ui
        self.ui.setupUi(self)
        # palettes that will be used to set the colors of the password strenght
        original_palette = self.ui.strenght_frame.palette()
        self._original_color = original_palette.background().color()
        self.next = -1
        self.captcha_id = None
        self.captcha_file = None
        self.controller = controller
        self.controller.setupUi(self)

    def set_strenght_level(self, level, password):
        """Set the strenght level colors."""
        if password != '':
            if level <= 1:
                # low password
                self._set_frame_color(self.ui.weak_frame, WEAK_COLOR)
                self._set_frame_color(self.ui.medium_frame,
                                      self._original_color)
                self._set_frame_color(self.ui.strong_frame,
                                      self._original_color)
            elif level <= 3:
                # medium password
                self._set_frame_color(self.ui.weak_frame, MEDIUM_COLOR)
                self._set_frame_color(self.ui.medium_frame, MEDIUM_COLOR)
                self._set_frame_color(self.ui.strong_frame,
                                      self._original_color)
            else:
                # nice!
                self._set_frame_color(self.ui.weak_frame, STRONG_COLOR)
                self._set_frame_color(self.ui.medium_frame, STRONG_COLOR)
                self._set_frame_color(self.ui.strong_frame, STRONG_COLOR)
        else:
            # set it to the minimum
            self._set_frame_color(self.ui.weak_frame, self._original_color)
            self._set_frame_color(self.ui.medium_frame, self._original_color)
            self._set_frame_color(self.ui.strong_frame, self._original_color)

    def _set_frame_color(self, frame, color):
        """Set the color of a frame to indicated a strenght."""
        # change the color background for the given frame
        palette = frame.palette()
        palette.setColor(QPalette.Background, color)
        frame.setPalette(palette)
        frame.setAutoFillBackground(True)

    # pylint: disable=C0103
    def nextId(self):
        """Return the next page id."""
        return self.next
    # pylint: enable=C0103

    def set_line_edit_validation_rule(self, edit, cb):
        """Set a new enhanced edit so that we can show an icon."""
        if edit in self._enhanced_edits:
            self._enhanced_edits[edit].valid_cb = cb
        else:
            # create a new enhanced edit
            enhanced_edit = EnhancedLineEdit(edit, cb)
            self._enhanced_edits[edit] = enhanced_edit

    @property
    def name(self):
        """Return the name input."""
        return str(self.ui.first_name_edit.text())

    @property
    def name_edit(self):
        """Return the edit used for the name."""
        return self.ui.first_name_edit

    @property
    def last_name(self):
        """Return the last name input."""
        return str(self.ui.last_name_edit.text())

    @property
    def last_name_edit(self):
        """Return the edit used for the last name."""
        return self.ui.last_name_edit

    @property
    def email(self):
        """Return the email input."""
        return str(self.ui.email_edit.text())

    @property
    def email_edit(self):
        """Return the edit used for the email."""
        return self.ui.email_edit

    @property
    def retyped_email(self):
        """Return the retyped email."""
        return str(self.ui.confirm_email_edit.text())

    @property
    def confirm_email_edit(self):
        """Return the edit used for the retype of the email."""
        return self.ui.confirm_email_edit

    @property
    def password_info_label(self):
        """Return the password used to show the info of the label."""
        return self.ui.password_info_label

    @property
    def password(self):
        """Return the password data."""
        return str(self.ui.password_edit.text())

    @property
    def password_edit(self):
        """Return the edit used for the password."""
        return self.ui.password_edit

    @property
    def retyped_password(self):
        """Return the retyped password."""
        return str(self.ui.confirm_password_edit.text())

    @property
    def confirm_password_edit(self):
        """Return the edit used to confirm the password."""
        return self.ui.confirm_password_edit

    @property
    def captcha_refresh_label(self):
        """Return the refresh label."""
        return self.ui.refresh_label

    @property
    def captcha_solution(self):
        """Return the provided captcha solution."""
        return str(self.ui.captcha_solution_edit.text())

    @property
    def captcha_solution_edit(self):
        """Return the edit used for the captcha solution."""
        return self.ui.captcha_solution_edit

    def get_captcha_image(self):
        """Return the path to the captcha image."""
        return self.captcha_id

    def set_captcha_image(self, data):
        """Set the new image of the captcha."""
        self.captcha_id = data
        # lets set the QPixmap for the label
        pic = QPixmap(self.captcha_file)
        self.ui.captcha_view.setPixmap(pic)

    captcha_image = property(get_captcha_image, set_captcha_image)

    @property
    def terms_and_conditions_agreed(self):
        """Return if the user agreed the terms."""
        return self.ui.terms_checkbox.isChecked()

    @property
    def terms_and_conditions_check_box(self):
        """Return the terms and codition item."""
        return self.ui.terms_checkbox

    @property
    def terms_and_conditions_button(self):
        """Return the terms and conditions button."""
        return self.ui.terms_button

    @property
    def set_up_button(self):
        """Return the set up button."""
        return self.ui.set_up_button


class UbuntuSSOWizard(QWizard):
    """Wizard used to create or use sso."""

    def __init__(self, parent=None, app_name='', tos_url='', help_text=''):
        """Create a new wizard."""
        QWizard.__init__(self, parent)
        # set the diff pages of the QWizard
        self.sign_in_controller = ChooseSignInController(title='Sign In',
                                                         existing_account_id=4,
                                                         new_account_id=1)
        self.sign_in_page = ChooseSignInPage(Ui_ChooseSignInPage(),
                                             self.sign_in_controller,
                                             parent=self)
        self.setup_controller = SetUpAccountController(tos_id=2,
                                                       validation_id=3,
                                                       app_name=app_name,
                                                       help_message='')
        self.setup_account = SetupAccountPage(Ui_SetUpAccountPage(),
                                              self.setup_controller,
                                              parent=self)
        self.tos = TosPage(Ui_TosPage(), TosController())
        self.email_verification = EmailVerificationPage(
                                                Ui_EmailVerificationPage(),
                                                EmailVerificationController())
        self.current_user_controller = CurrentUserController(title='Sign in',
                                                            app_name=app_name)
        self.current_user = CurrentUserSignInPage(Ui_CurrentUserSignInPage(),
                                                  self.current_user_controller,
                                                  parent=self)
        for page in [self.sign_in_page, self.setup_account, self.tos,
                     self.email_verification, self.current_user]:
            self.addPage(page)
        # set the buttons layout to only have cancel and back since the next
        # buttons are the ones used in the diff pages.
        buttons_layout = []
        buttons_layout.append(QWizard.Stretch)
        buttons_layout.append(QWizard.BackButton)
        buttons_layout.append(QWizard.CancelButton)
        self.setButtonLayout(buttons_layout)
        self.setWindowTitle(app_name)


class UbuntuSSOWizardController(object):
    """Performs the logic of the sso wizard."""

    def __init__(self, app_name, tc_url='', help_text='',
                 window_id=0, login_only=False):
        """Create a new instance."""
       # create a new wizard and show it
        self.wizard = None


class UbuntuSSOClientGUI(object):
    """Ubuntu single sign-on GUI."""
