# -*- coding: utf-8 -*-
# Author: Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Test the ui controllers."""

from mocker import ANY, MATCH, MockerTestCase

from ubuntu_sso.qt.controllers import (
    ChooseSignInController,
    CurrentUserController,
    EmailVerificationController,
    SetUpAccountController,
    TosController)
from ubuntu_sso.utils.ui import (
    CAPTCHA_SOLUTION_ENTRY,
    EMAIL1_ENTRY,
    EMAIL2_ENTRY,
    EMAIL_MISMATCH,
    EXISTING_ACCOUNT_CHOICE_BUTTON,
    FORGOTTEN_PASSWORD_BUTTON,
    JOIN_HEADER_LABEL,
    NAME_ENTRY,
    PASSWORD1_ENTRY,
    PASSWORD2_ENTRY,
    PASSWORD_HELP,
    PASSWORD_TOO_WEAK,
    PASSWORD_MISMATCH,
    SET_UP_ACCOUNT_CHOICE_BUTTON,
    SET_UP_ACCOUNT_BUTTON,
    SIGN_IN_BUTTON,
    SURNAME_ENTRY,
    TC_BUTTON,
    VERIFY_EMAIL_TITLE,
    VERIFY_EMAIL_CONTENT,
    YES_TO_TC,
    is_min_required_password)

#ignore the comon mocker issues with lint
# pylint: disable=W0212,W0104


class ChooseSignInControllerTestCase(MockerTestCase):
    """Test the choose sign in controller."""

    def setUp(self):
        """Set tests."""
        super(ChooseSignInControllerTestCase, self).setUp()
        self.view = self.mocker.mock()
        self.controller = ChooseSignInController()

    def test_set_up_translated_strings(self):
        """Ensure that the translations are used."""
        self.view.existing_account_button.setText(
                                            EXISTING_ACCOUNT_CHOICE_BUTTON)
        self.view.new_account_button.setText(SET_UP_ACCOUNT_CHOICE_BUTTON)
        self.mocker.replay()
        self.controller._set_up_translated_strings(self.view)

    def test_connect_buttons(self):
        """Ensure that all the buttons are correcly connected."""
        self.view.existing_account_button.clicked.connect(MATCH(callable))
        self.view.new_account_button.clicked.connect(MATCH(callable))
        self.mocker.replay()
        self.controller._connect_buttons(self.view)

    def test_set_next_existing(self):
        """Test the execution of the callback."""
        self.view.next = self.controller._existing_account_id
        self.view.wizard().next()
        self.mocker.replay()
        self.controller._set_next_existing(self.view)

    def test_set_next_new(self):
        """Test the execution of the callback."""
        self.view.next = self.controller._new_account_id
        self.view.wizard().next()
        self.mocker.replay()
        self.controller._set_next_new(self.view)


class CurrentUserControllerTestCase(MockerTestCase):
    """Test the current user controller."""

    def setUp(self):
        """Setup tests."""
        super(CurrentUserControllerTestCase, self).setUp()
        self.view = self.mocker.mock()
        self.backend = self.mocker.mock()
        self.connect = self.mocker.mock()
        self.signal = self.mocker.replace('PyQt4.QtCore.SIGNAL')
        self.controller = CurrentUserController(title='the title',
                                                backend=self.backend)

    def test_translated_strings(self):
        """test that the ui is correctly set up."""
        self.view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        self.view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        self.view.forgot_password_label.setText(FORGOTTEN_PASSWORD_BUTTON)
        self.view.sign_in_button.setText(SIGN_IN_BUTTON)
        self.mocker.replay()
        self.controller._set_translated_strings(self.view)

    def test_connect_ui(self):
        """test that the ui is correctly set up."""
        self.view.sign_in_button.clicked.connect(MATCH(callable))
        self.backend.on_login_error_cb = MATCH(callable)
        self.backend.on_logged_in_cb = MATCH(callable)
        self.mocker.replay()
        self.controller._connect_buttons(self.view, self.backend)


class SetUpAccountControllerTestCase(MockerTestCase):
    """test the controller used to setup a new account."""

    def setUp(self):
        """Set the different tests."""
        super(SetUpAccountControllerTestCase, self).setUp()
        self.view = self.mocker.mock()
        self.get_password_strength = self.mocker.replace(
                                 'ubuntu_sso.utils.ui.get_password_strength')
        self.app_name = 'test'
        self.help = 'help'
        self.message_box = self.mocker.mock()
        self.controller = SetUpAccountController(app_name=self.app_name,
                                                 help_message=self.help,
                                                 message_box=self.message_box)

    def test_set_translated_strings(self):
        """Ensure all the strings are set."""
        self.view.name_edit.setPlaceholderText(NAME_ENTRY)
        self.view.last_name_edit.setPlaceholderText(SURNAME_ENTRY)
        self.view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        self.view.confirm_email_edit.setPlaceholderText(EMAIL2_ENTRY)
        self.view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        self.view.confirm_password_edit.setPlaceholderText(PASSWORD2_ENTRY)
        self.view.password_info_label.setText(PASSWORD_HELP)
        self.view.captcha_solution_edit.setPlaceholderText(
                                                    CAPTCHA_SOLUTION_ENTRY)
        self.view.terms_and_conditions_check_box.setText(YES_TO_TC)
        self.view.terms_and_conditions_button.setText(TC_BUTTON)
        self.view.set_up_button.setText(SET_UP_ACCOUNT_BUTTON)
        self.mocker.replay()
        self.controller._set_translated_strings(self.view)

    def test_set_titles(self):
        """Test how the different titles are set."""
        self.view.setTitle(JOIN_HEADER_LABEL % {'app_name': self.app_name})
        self.view.setSubTitle(self.help)
        self.mocker.replay()
        self.controller._set_titles(self.view)

    def test_connect_ui_elements(self):
        """Test that the ui elements are correctly connect."""
        backend = self.mocker.mock()
        self.view.terms_and_conditions_button.clicked.connect(MATCH(callable))
        self.view.set_up_button.clicked.connect(MATCH(callable))
        self.view.password_edit.textChanged.connect(MATCH(callable))
        self.view.set_up_button.setEnabled
        self.mocker.result(lambda: None)
        self.view.terms_and_conditions_check_box.stateChanged.connect(
                                                             MATCH(callable))
        self.view.captcha_refresh_label.linkActivated.connect(MATCH(callable))
        # set the callbacks for the captcha generatio
        backend.on_captcha_generated_cb = MATCH(callable)
        backend.on_captcha_generation_error_cb = MATCH(callable)
        backend.on_user_registration_error_cb = MATCH(callable)
        self.mocker.replay()
        self.controller._connect_ui_elements(self.view, backend)

    def test_is_correct_password_confirmation_false(self):
        """Test when the password is not correct."""
        password = 'test'
        self.view.password_edit.text()
        self.mocker.result('other')
        self.mocker.replay()
        self.assertFalse(
                    self.controller.is_correct_password_confirmation(password,
                                                                   self.view))

    def test_is_correct_password_confirmation_true(self):
        """Test when the password is correct."""
        password = 'test'
        self.view.password_edit.text()
        self.mocker.result(password)
        self.mocker.replay()
        self.assertTrue(
                    self.controller.is_correct_password_confirmation(password,
                                                                   self.view))

    def test_is_correct_email_confirmation_false(self):
        """Test when the emails are not the same."""
        email_address = 'address'
        self.view.email_edit.text()
        self.mocker.result('other')
        self.mocker.replay()
        self.assertFalse(
                self.controller.is_correct_email_confirmation(email_address,
                                                              self.view))

    def test_is_correct_email_confirmation_true(self):
        """Test when the emails are the same."""
        email_address = 'address'
        self.view.email_edit.text()
        self.mocker.result(email_address)
        self.mocker.replay()
        self.assertTrue(
                self.controller.is_correct_email_confirmation(email_address,
                                                              self.view))

    def test_is_correct_email_true(self):
        """Test when the email is correct."""
        email = 'manuel@canonical.com'
        self.mocker.replay()
        self.assertTrue(self.controller.is_correct_email(email))

    def test_is_correct_email_false(self):
        """Test when the email is not correct."""
        email = 'manuelcanonical.com'
        self.mocker.replay()
        self.assertFalse(self.controller.is_correct_email(email))

    def test_set_next_tos(self):
        """Test the callback."""
        self.view.next = self.controller._tos_id
        self.view.wizard().next()
        self.mocker.replay()
        self.controller.set_next_tos(self.view)

    def test_set_next_validation(self):
        """Test the callback."""
        email = 'email@example.com'
        password = 'Qwerty9923'
        captcha_id = 'captcha_id'
        captcha_solution = 'captcha_solution'
        self.view.email
        self.mocker.result(email)
        self.view.email_edit.text()
        self.mocker.result(email)
        self.view.confirm_email_edit.text()
        self.mocker.result(email)
        self.view.password_edit.text()
        self.mocker.result(password)
        self.view.password_edit.text()
        self.mocker.result(password)
        self.view.confirm_password_edit.text()
        self.mocker.result(password)
        self.view.email
        self.mocker.result(email)
        self.view.password
        self.mocker.result(password)
        self.view.captcha_id
        self.mocker.result(captcha_id)
        self.view.captcha_solution
        self.mocker.result(captcha_solution)
        backend = self.mocker.mock()
        backend.register_user(self.app_name, email, password, captcha_id,
                              captcha_solution)
        self.view.next = self.controller._validation_id
        self.view.wizard().next()
        self.mocker.replay()
        self.controller.set_next_validation(self.view, backend)

    def test_set_next_validation_wrong_email(self):
        """Test the callback when there is a wrong email."""
        email = 'email@example.com'
        self.view.email
        self.mocker.result(email)
        self.view.email_edit.text()
        self.mocker.result('email')
        self.view.confirm_email_edit.text()
        self.mocker.result('email2')
        self.message_box.critical(self.view, self.app_name, EMAIL_MISMATCH)
        self.mocker.replay()
        self.assertFalse(self.controller.validate_form(self.view))

    def test_set_next_validation_not_min_password(self):
        """Test the callback with a weak password."""
        weak_password = 'weak'
        email = 'email@example.com'
        self.view.email
        self.mocker.result(email)
        self.view.email_edit.text()
        self.mocker.result(email)
        self.view.confirm_email_edit.text()
        self.mocker.result(email)
        self.view.password_edit.text()
        self.mocker.result(weak_password)
        self.message_box.critical(self.view, self.app_name, PASSWORD_TOO_WEAK)
        self.mocker.replay()
        self.assertFalse(self.controller.validate_form(self.view))

    def test_set_next_validation_wrong_password(self):
        """Test the callback where the password is wrong."""
        password = 'Qwerty9923'
        email = 'email@example.com'
        self.view.email
        self.mocker.result(email)
        self.view.email_edit.text()
        self.mocker.result(email)
        self.view.confirm_email_edit.text()
        self.mocker.result(email)
        self.view.password_edit.text()
        self.mocker.result(password)
        self.view.password_edit.text()
        self.mocker.result(password)
        self.view.confirm_password_edit.text()
        self.mocker.result('other_password')
        self.message_box.critical(self.view, self.app_name, PASSWORD_MISMATCH)
        self.mocker.replay()
        self.assertFalse(self.controller.validate_form(self.view))

    def test_update_password_strength(self):
        """Test the callback."""
        password = 'test'
        strength = 4
        self.get_password_strength(password)
        self.mocker.result(strength)
        self.view.set_strenght_level(strength, password)
        self.mocker.replay()
        self.controller.update_password_strength(password, self.view)

    def test_set_line_edits_validations(self):
        """Set the validations to be performed on the edits."""
        self.view.email_edit
        self.mocker.result(self.view)
        self.view.set_line_edit_validation_rule(self.view,
                                            self.controller.is_correct_email)
        self.view.confirm_email_edit
        self.mocker.result(self.view)
        self.view.set_line_edit_validation_rule(self.view, MATCH(callable))
        self.view.confirm_email_edit.textChanged.emit
        self.mocker.result(lambda: None)
        self.view.email_edit.textChanged.connect(MATCH(callable))
        self.view.password_edit
        self.mocker.result(self.view)
        self.view.set_line_edit_validation_rule(self.view,
                                                is_min_required_password)
        self.view.confirm_password_edit
        self.mocker.result(self.view)
        self.view.set_line_edit_validation_rule(self.view, MATCH(callable))
        self.view.confirm_password_edit.textChanged.emit
        self.mocker.result(lambda: None)
        self.view.password_edit.textChanged.connect(MATCH(callable))
        self.mocker.replay()
        self.controller._set_line_edits_validations(self.view)


class TosControllerTestCase(MockerTestCase):
    """Test the tos controller."""

    def setUp(self):
        """Set the tests."""
        super(TosControllerTestCase, self).setUp()
        self.view = self.mocker.mock()
        self.title = 'title'
        self.subtitle = 'sub'
        self.url = 'url'
        self.controller = TosController(title=self.title,
                                        subtitle=self.subtitle,
                                        tos_url=self.url)

    def test_setup_ui(self):
        """Test the set up of the ui."""
        self.view.setTitle(self.title)
        self.view.setSubTitle(self.subtitle)
        self.view.webkit.load(ANY)
        self.mocker.replay()
        self.controller.setupUi(self.view)


class EmailVerificationControllerTestCase(MockerTestCase):
    """Test the controller."""

    def setUp(self):
        """Set tests."""
        super(EmailVerificationControllerTestCase, self).setUp()
        self.view = self.mocker.mock()
        self.controller = EmailVerificationController()

    def test_set_translated_strings(self):
        """Test that strings are translated."""
        self.view.verification_code_edit.setPlaceholderText(VERIFY_EMAIL_TITLE)
        self.mocker.replay()
        self.controller._set_translated_strings(self.view)

    def test_connect_ui_elements(self):
        """Set the ui connections."""
        self.view.next_button.clicked.connect(MATCH(callable))
        self.mocker.replay()
        self.controller._connect_ui_elements(self.view)

    def test_set_titles(self):
        """Test that the titles are set."""
        self.view.setTitle(VERIFY_EMAIL_TITLE)
        self.view.setSubTitle(VERIFY_EMAIL_CONTENT)
        self.mocker.replay()
        self.controller._set_titles(self.view)

    def test_next_page(self):
        """Test the callback."""
        self.view.wizard().next()
        self.mocker.replay()
        self.controller.next_page(self.view)
